unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MMSystem, ExtCtrls, ComCtrls, DSPack, 
  OleCtrls, SHDocVw, AppEvnts, Contnrs,
  DSUtil, ActiveX, DirectShow9;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    CloseButton: TLabel;
    Label1: TLabel;
    Timer1: TTimer;
    FilterGraph: TFilterGraph;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FilterGraphGraphVideoSizeChanged(sender: TObject; Width,
      height: Word);
    procedure FilterGraphDSEvent(sender: TComponent; Event, Param1,
      Param2: Integer);
    function FilterGraphSelectedFilter(Moniker: IMoniker;
      FilterName: WideString; ClassID: TGUID): Boolean;
    procedure Timer1Timer(Sender: TObject);
  end;
  
var
  Form1: TForm1;
  DebugFile: string;
  doDebug: boolean;

implementation

{$R *.DFM}

uses shellApi, {DSPlayerAsyncSourceTypes, {}Buttons, 
     math, {Filter, {}Unit2, utils;

var
  filename:string;

procedure debug(s:string);
var f:textfile;
begin
  if Dodebug then
  begin
    assignfile(f,DebugFile);
    try
      append(f);
    except
      rewrite(f);
    end;
    writeln(f,'['+datetimetostr(now)+'] '+s);
    closefile(f);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
const MinH=45;
var
  s:string;

  w,h,tt:integer;

  isVideoVisible:longbool;

  BasicVideo : IBasicVideo;
  VideoWindow : IVideoWindow;
  FilterList : TFilterList;
  BaseFilter : IBaseFilter;
  pi : _FilterInfo;
  fg : IFilterGraph;
  IdealWidth  : Integer;
  IdealHeight : Integer;

  function tryDirectShow:boolean;
  var i : integer;
  begin
    debug('trying direct show...');
//    result:=false;
    try
//      VideoWindow1.Show;
//      DSTrackBar1.Show;
      if not FilterGraph.Active then FilterGraph.Active := true;
      FilterGraph.ClearGraph;
      FilterGraph.Volume:=10000;
      FilterGraph.RenderFile(filename);
      FilterGraph.Play;

      FilterGraph.QueryInterface(IVideoWindow, VideoWindow);
      form2.show;
      if VideoWindow<>nil then
      begin
        videoWindow.put_Owner(form2.Handle);
        VideoWindow.get_Visible(isVideoVisible);
        VideoWindow.put_Caption('My Async Player');
        VideoWindow.put_Left(0);
        VideoWindow.put_Top(0);
        VideoWindow.put_WindowStyle(WS_DLGFRAME);
        VideoWindow.put_AutoShow(true);
      end;

      FilterGraph.QueryInterface(IBasicVideo, BasicVideo);
      if BasicVideo<>nil then
      begin
        BasicVideo.GetVideoSize(IdealWidth, IdealHeight);
        w:=IdealWidth;
        h:=Idealheight;
      end                else
      begin
        w:=0;
        h:=0;
      end;
      if (w>0) and (h>0) then
      begin
//        if w<382 then w:=382;
//        form2.ClientWidth:=w;
//        form2.ClientHeight:=h;//+MinH;
      end                else
      begin
//        form2.ClientHeight:=MinH;// playing sound
//        panel1.caption:='Sound is playing...';
      end;                (**)
      debug('finished setting up directshow');
      try
        debug('starting to enumerate used filters...');
        FilterGraph.QueryInterface(IID_IFilterGraph, fg);
        if assigned(fg) then
        begin
          FilterList:= TFilterList.Create(fg);
          for i := 0 to FilterList.Count - 1 do
          begin
            BaseFilter := FilterList.Items[i] as IBaseFilter;
            BaseFilter.QueryFilterInfo(pi);
            debug('found filter: ' + pi.achName);
            BaseFilter := nil;
          end;
          FilterList.Free;
        end             else
          debug('queryInterface for IFilterGraph failed');
        debug('finish retrieveing filters');
      except
      end;
    except
      on e:exception do
      begin
        debug('EXCEPTION: '+e.message);
//        ShowMessage('Exception ocured using DirectShow: '+e.message);
//      VideoWindow1.Hide;
//      DSTrackBar1.Hide;
        FilterGraph.ClearGraph;
        FilterGraph.Active := false;
      end;
    end;
    result:=true;
  end;

var i:integer;
begin
  randomize;

  DebugFile:='';

  if paramcount>0 then
  begin
    doDebug:=false;
    for i:=1 to paramcount do
    begin
      if lowercase(paramstr(i))='/debug' then
        doDebug:=true;
      if lowercase(paramstr(i))='/local' then
        DebugFile:=' ';// current dir
    end;
  end;

  if DebugFile=' ' then DebugFile:='debug - '+extractfilename(application.ExeName)+'.log'
                   else DebugFile:=getTempDir+'debug - '+extractfilename(application.ExeName)+'.log';

  deleteFile(DebugFile);                 
  debug('loading...');

try
  if not OpenDialog1.Execute then
    exit;
  filename:=OpenDialog1.FileName;

  tryDirectShow;

  left:=0;
  tt:=0;
  VideoWindow.get_Height(tt);
  top:=1+tt+3;
  if top+Height>Screen.WorkAreaHeight then
    top:=Screen.WorkAreaHeight-Height-1;

  debug('loading finished');
  timer1.enabled:=true;
  
except
  on e:exception do
  begin
    s:='Exception occured when initializing: '+e.message;
    debug(s);
    showmessage(s);
    application.terminate;
  end;
end;
  debug('loading done.');
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  // Important
  FilterGraph.ClearGraph;
end;

procedure TForm1.FilterGraphGraphVideoSizeChanged(sender: TObject; Width,
  height: Word);
var h,w:integer;
begin
  h:=height;
  w:=width;
  if (w>0) and (h>0) then
  begin
//    if w<382 then w:=382;
//    ClientWidth:=w;
//    ClientHeight:=h;
  end                else
  begin
//    ClientHeight:=28;// playing sound
//    panel1.caption:='Sound is playing...';
  end;
end;

procedure TForm1.FilterGraphDSEvent(sender: TComponent; Event, Param1,
  Param2: Integer);
begin
  case event of
    EC_COMPLETE:close;//over:=true;
  end;
end;

//  debug('created filter with ID: '+GUIDToString(classID));

function TForm1.FilterGraphSelectedFilter(Moniker: IMoniker;
  FilterName: WideString; ClassID: TGUID): Boolean;
begin
  debug('selected filter with name: '+FilterName+' and ID: '+GUIDToString(classID));
  result:=true;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  timer1.enabled:=false;
  SetForegroundWindow(handle);
  ShowWindow(handle, 9);
end;

end.
